@@ -1,21 +1,18 @@
-__author__ = 'Copyright (c) 2013 Alan Yorinks All rights reserved.'
-
 """
-Copyright (c) 2013-14 Alan Yorinks All rights reserved.
+ Copyright (c) 2015-2017 Alan Yorinks All rights reserved.
 
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU  General Public
-License as published by the Free Software Foundation; either
-version 3 of the License, or (at your option) any later version.
+ This program is free software; you can redistribute it and/or
+ modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
+ Version 3 as published by the Free Software Foundation; either
+ or (at your option) any later version.
+ This library is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ General Public License for more details.
 
-This library is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-General Public License for more details.
-
-You should have received a copy of the GNU General Public
-License along with this library; if not, write to the Free Software
-Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ You should have received a copy of the GNU AFFERO GENERAL PUBLIC LICENSE
+ along with this library; if not, write to the Free Software
+ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 """
 
 import threading
@@ -35,7 +32,6 @@ class PyMataCommandHandler(threading.Thread):
 
     There is blocking when accessing the data tables through the _data_lock
     """
-
     # the following defines are from Firmata.h
 
     # message command bytes (128-255/ 0x80- 0xFF)
@@ -54,19 +50,19 @@ class PyMataCommandHandler(threading.Thread):
     REPORT_VERSION = 0xF9  # report protocol version
 
     # user defined SYSEX commands
-    ENCODER_CONFIG = 0x20  # create and enable encoder object
+    ENCODER_CONFIG = 0x60  # create and enable encoder object
     TONE_PLAY = 0x5F  # play a tone at a specified frequency and duration
-    SONAR_CONFIG = 0x60  # configure pins to control a Ping type sonar distance device
+    SONAR_CONFIG = 0x62  # configure pins to control a Ping type sonar distance device
 
-    ENCODER_DATA = 0x21  # current encoder position data
-    SONAR_DATA = 0x61  # distance data returned
+    ENCODER_DATA = 0x61  # current encoder position data
+    SONAR_DATA = 0x63  # distance data returned
 
     SERVO_CONFIG = 0x70  # set servo pin and max and min angles
     STRING_DATA = 0x71  # a string message with 14-bits per char
     STEPPER_DATA = 0x72  # Stepper motor command
     I2C_REQUEST = 0x76  # send an I2C read/write request
-    I2C_REPLY = 0x77    # a reply to an I2C read request
-    I2C_CONFIG = 0x78   # config I2C settings such as delay times and power pins
+    I2C_REPLY = 0x77  # a reply to an I2C read request
+    I2C_CONFIG = 0x78  # config I2C settings such as delay times and power pins
     REPORT_FIRMWARE = 0x79  # report name and version of the firmware
     SAMPLING_INTERVAL = 0x7A  # modify the sampling interval
 
@@ -83,7 +79,8 @@ class PyMataCommandHandler(threading.Thread):
     SYSEX_REALTIME = 0x7F  # MIDI Reserved for realtime messages
 
     # The response tables hold response information for all pins
-    # Each table is a table of entries for each pin, which consists of the pin mode, and its last value from firmata
+    # Each table is a table of entries for each pin, which consists of the pin mode, its last value from firmata
+    # and a callback function that the user attached to the pin
 
     # This is a table that stores analog pin modes and data
     # each entry represents ia mode (INPUT or OUTPUT), and its last current value
@@ -114,11 +111,13 @@ class PyMataCommandHandler(threading.Thread):
     ANALOG_LATCH_DATA_TARGET = 2
     ANALOG_LATCHED_DATA = 3
     ANALOG_TIME_STAMP = 4
+    ANALOG_LATCH_CALLBACK = 5
 
     DIGITAL_LATCHED_DATA = 2
     DIGITAL_TIME_STAMP = 3
+    DIGITAL_LATCH_CALLBACK = 4
 
-    #latch states
+    # latch states
     LATCH_IGNORE = 0  # this pin will be ignored for latching
     LATCH_ARMED = 1  # When the next pin value change is received for this pin, if it matches the latch criteria
     # the data will be latched
@@ -135,6 +134,7 @@ class PyMataCommandHandler(threading.Thread):
     # These values are indexes into the response table entries
     RESPONSE_TABLE_MODE = 0
     RESPONSE_TABLE_PIN_DATA_VALUE = 1
+    RESPONSE_TABLE_CALLBACK = 2
 
     # These values are the index into the data passed by _arduino and used to reassemble integer values
     MSB = 2
@@ -167,10 +167,17 @@ class PyMataCommandHandler(threading.Thread):
     # total number of analog pins for the discovered board
     number_of_analog_pins_discovered = 0
 
-    # map of i2c addresses and associated data that has been read for that address
+    # The i2c_map will contain keys of i2c device addresses, and an associated list.
+    # The associated list will contain 2 elements:
+    # 1. A callback reference. This reference will be set to None if no callback was registered.
+    # 2. Data returned from a an i2c read request.
+
     i2c_map = {}
 
     # the active_sonar_map maps the sonar trigger pin number (the key) to the current data value returned
+    # if a callback was specified, it is stored in the map as well.
+    # an entry in the map consists of:
+    #   pin: [callback,[current_data_returned]]
     active_sonar_map = {}
 
     # the stepper library version number.
@@ -179,7 +186,8 @@ class PyMataCommandHandler(threading.Thread):
     def __init__(self, pymata):
         """
         constructor for CommandHandler class
-        @param pymata: A reference to the pymata instance.
+        
+        :param pymata: A reference to the pymata instance.
         """
 
         # reference pointer to pymata
@@ -200,7 +208,7 @@ class PyMataCommandHandler(threading.Thread):
 
         threading.Thread.__init__(self)
         self.daemon = True
-        
+
         self.stop_event = threading.Event()
 
     def stop(self):
@@ -209,11 +217,11 @@ class PyMataCommandHandler(threading.Thread):
     def is_stopped(self):
         return self.stop_event.is_set()
 
-    def auto_discover_board(self):
+    def auto_discover_board(self, verbose):
         """
         This method will allow up to 30 seconds for discovery (communicating with) an Arduino board
         and then will determine a pin configuration table for the board.
-        @return: True if board is successfully discovered or False upon timeout
+        :return: True if board is successfully discovered or False upon timeout
         """
         # get current time
         start_time = time.time()
@@ -224,10 +232,11 @@ class PyMataCommandHandler(threading.Thread):
             if time.time() - start_time > 30:
                 return False
                 # keep sending out a capability query until there is a response
-            self.send_sysex(self.ANALOG_MAPPING_QUERY, None)
+            self.send_sysex(self.ANALOG_MAPPING_QUERY)
             time.sleep(.1)
 
-        print "Board initialized in %d seconds" % (time.time() - start_time)
+        if verbose:
+            print("Board initialized in %d seconds" % (time.time() - start_time))
 
         for pin in self.analog_mapping_query_results:
             self.total_pins_discovered += 1
@@ -235,26 +244,27 @@ class PyMataCommandHandler(threading.Thread):
             if pin != self.pymata.IGNORE:
                 self.number_of_analog_pins_discovered += 1
 
-        print 'Total Number of Pins Detected = %d' % self.total_pins_discovered
-        print 'Total Number of Analog Pins Detected = %d' % self.number_of_analog_pins_discovered
+        if verbose:
+            print('Total Number of Pins Detected = %d' % self.total_pins_discovered)
+            print('Total Number of Analog Pins Detected = %d' % self.number_of_analog_pins_discovered)
 
         # response table initialization
         # for each pin set the mode to input and the last read data value to zero
         for pin in range(0, self.total_pins_discovered):
-            response_entry = [self.pymata.INPUT, 0]
+            response_entry = [self.pymata.INPUT, 0, None]
             self.digital_response_table.append(response_entry)
 
         for pin in range(0, self.number_of_analog_pins_discovered):
-            response_entry = [self.pymata.INPUT, 0]
+            response_entry = [self.pymata.INPUT, 0, None]
             self.analog_response_table.append(response_entry)
 
         # set up latching tables
         for pin in range(0, self.total_pins_discovered):
-            digital_latch_table_entry = [0, 0, 0, 0]
+            digital_latch_table_entry = [0, 0, 0, 0, None]
             self.digital_latch_table.append(digital_latch_table_entry)
 
         for pin in range(0, self.number_of_analog_pins_discovered):
-            analog_latch_table_entry = [0, 0, 0, 0, 0]
+            analog_latch_table_entry = [0, 0, 0, 0, 0, None]
             self.analog_latch_table.append(analog_latch_table_entry)
 
         return True
@@ -265,48 +275,62 @@ class PyMataCommandHandler(threading.Thread):
         or after refresh_report_version() is called
 
         Use the api method api_get_version to retrieve this information
-        @param data: Message data from Firmata
-        @return: No return value.
+        
+        :param data: Message data from Firmata
+        
+        :return: No return value.
         """
         self.firmata_version.append(data[0])  # add major
         self.firmata_version.append(data[1])  # add minor
 
-    def set_analog_latch(self, pin, threshold_type, threshold_value):
+    def set_analog_latch(self, pin, threshold_type, threshold_value, cb):
         """
         This method "arms" a pin to allow data latching for the pin.
-        @param pin: Analog pin number (value following an 'A' designator, i.e. A5 = 5
-        @param threshold_type: ANALOG_LATCH_GT | ANALOG_LATCH_LT  | ANALOG_LATCH_GTE | ANALOG_LATCH_LTE
-        @param threshold_value: numerical value
-        """
-        with self.pymata.data_lock:
-            self.analog_latch_table[pin] = [self.LATCH_ARMED, threshold_type, threshold_value, 0, 0]
 
-    def set_digital_latch(self, pin, threshold_type):
-        """
-        This method "arms" a pin to allow data latching for the pin.
-        @param pin: digital pin number
-        @param threshold_type: DIGITAL_LATCH_HIGH | DIGITAL_LATCH_LOW
+        :param pin: Analog pin number (value following an 'A' designator, i.e. A5 = 5
+
+        :param threshold_type: ANALOG_LATCH_GT | ANALOG_LATCH_LT  | ANALOG_LATCH_GTE | ANALOG_LATCH_LTE
+
+        :param threshold_value: numerical value
+
+        :param cb: User provided callback function
         """
         with self.pymata.data_lock:
-            self.digital_latch_table[pin] = [self.LATCH_ARMED, threshold_type, 0, 0]
+            self.analog_latch_table[pin] = [self.LATCH_ARMED, threshold_type, threshold_value, 0, 0, cb]
+
+    def set_digital_latch(self, pin, threshold_type, cb):
+        """
+        This method "arms" a pin to allow data latching for the pin.
+
+        :param pin: digital pin number
+
+        :param threshold_type: DIGITAL_LATCH_HIGH | DIGITAL_LATCH_LOW
+
+        :param cb: User provided callback function
+        """
+        with self.pymata.data_lock:
+            self.digital_latch_table[pin] = [self.LATCH_ARMED, threshold_type, 0, 0, cb]
 
     def get_analog_latch_data(self, pin):
         """
         This method reads the analog latch table for the specified pin and returns a list that contains:
         [latch_state, latched_data, and time_stamp].
         If the latch state is latched, the entry in the table is cleared
-        @param pin:  pin number
-        @return: [latch_state, latched_data, and time_stamp]
+
+        :param pin:  pin number
+
+        :return: [latch_state, latched_data, and time_stamp]
         """
         with self.pymata.data_lock:
             pin_data = self.analog_latch_table[pin]
             current_latch_data = [pin,
                                   pin_data[self.LATCH_STATE],
                                   pin_data[self.ANALOG_LATCHED_DATA],
-                                  pin_data[self.ANALOG_TIME_STAMP]]
+                                  pin_data[self.ANALOG_TIME_STAMP],
+                                  pin_data[self.ANALOG_LATCH_CALLBACK]]
             # if this is latched data, clear the latch table entry for this pin
             if pin_data[self.LATCH_STATE] == self.LATCH_LATCHED:
-                self.analog_latch_table[pin] = [0, 0, 0, 0, 0]
+                self.analog_latch_table[pin] = [0, 0, 0, 0, 0, None]
         return current_latch_data
 
     def get_digital_latch_data(self, pin):
@@ -314,17 +338,20 @@ class PyMataCommandHandler(threading.Thread):
         This method reads the digital latch table for the specified pin and returns a list that contains:
         [latch_state, latched_data, and time_stamp].
         If the latch state is latched, the entry in the table is cleared
-        @param pin:  pin number
-        @return: [latch_state, latched_data, and time_stamp]
+
+        :param pin:  pin number
+
+        :return: [latch_state, latched_data, and time_stamp]
         """
         with self.pymata.data_lock:
             pin_data = self.digital_latch_table[pin]
             current_latch_data = [pin,
                                   pin_data[self.LATCH_STATE],
                                   pin_data[self.DIGITAL_LATCHED_DATA],
-                                  pin_data[self.DIGITAL_TIME_STAMP]]
+                                  pin_data[self.DIGITAL_TIME_STAMP],
+                                  pin_data[self.DIGITAL_LATCH_CALLBACK]]
             if pin_data[self.LATCH_STATE] == self.LATCH_LATCHED:
-                self.digital_latch_table[pin] = [0, 0, 0, 0]
+                self.digital_latch_table[pin] = [0, 0, 0, 0, None]
         return current_latch_data
 
     def report_firmware(self, data):
@@ -333,8 +360,10 @@ class PyMataCommandHandler(threading.Thread):
         or after refresh_report_firmware() is called
         
         Use the api method api_get_firmware_version to retrieve this information
-        @param data: Message data from Firmata
-        @return: No return value.
+
+        :param data: Message data from Firmata
+
+        :return: No return value.
         """
         self.firmata_firmware.append(data[0])  # add major
         self.firmata_firmware.append(data[1])  # add minor
@@ -358,18 +387,32 @@ class PyMataCommandHandler(threading.Thread):
         """
         This method handles the incoming analog data message.
         It stores the data value for the pin in the analog response table.
-        It checks to see if the
-        @param data: Message data from Firmata
-        @return: No return value.
+        If a callback function was associated with this pin, the callback function is invoked.
+        This method also checks to see if latching was requested for the pin. If the latch criteria was met,
+        the latching table is updated. If a latching callback function was provided by the user, a latching
+        notification callback message is sent to the user in place of updating the latching table.
+
+        :param data: Message data from Firmata
+
+        :return: No return value.
         """
         with self.pymata.data_lock:
-            # convert MSB and LSB into an integer
+            # hold on to the previous value
+            previous_value = \
+                self.analog_response_table[data[self.RESPONSE_TABLE_MODE]][self.RESPONSE_TABLE_PIN_DATA_VALUE]
             self.analog_response_table[data[self.RESPONSE_TABLE_MODE]][self.RESPONSE_TABLE_PIN_DATA_VALUE] \
                 = (data[self.MSB] << 7) + data[self.LSB]
-
             pin = data[0]
             pin_response_data_data = self.analog_response_table[pin]
             value = pin_response_data_data[self.RESPONSE_TABLE_PIN_DATA_VALUE]
+            # check to see if there is a callback function attached to this pin
+            callback = self.analog_response_table[data[self.RESPONSE_TABLE_MODE]][self.RESPONSE_TABLE_CALLBACK]
+            # send the pin mode, pin number, and current data value
+            if callback is not None:
+                if value != previous_value:
+                    # has the value changed since the last report
+                    callback([self.pymata.ANALOG, pin, value])
+
             # check if data is to be latched
             # get the analog latching table entry for this pin
             latching_entry = self.analog_latch_table[pin]
@@ -377,42 +420,62 @@ class PyMataCommandHandler(threading.Thread):
                 # Has the latching criteria been met
                 if latching_entry[self.LATCHED_THRESHOLD_TYPE] == self.ANALOG_LATCH_GT:
                     if value > latching_entry[self.ANALOG_LATCH_DATA_TARGET]:
-                        updated_latch_entry = latching_entry
-                        updated_latch_entry[self.LATCH_STATE] = self.LATCH_LATCHED
-                        updated_latch_entry[self.ANALOG_LATCHED_DATA] = value
-                        # time stamp it
-                        updated_latch_entry[self.ANALOG_TIME_STAMP] = time.time()
-                        self.analog_latch_table[pin] = updated_latch_entry
+                        if latching_entry[self.ANALOG_LATCH_CALLBACK] is not None:
+                            self.analog_latch_table[pin] = [0, 0, 0, 0, 0, None]
+                            latching_entry[self.ANALOG_LATCH_CALLBACK]([self.pymata.ANALOG | self.pymata.LATCH_MODE,
+                                                                        pin, value, time.time()])
+                        else:
+                            updated_latch_entry = latching_entry
+                            updated_latch_entry[self.LATCH_STATE] = self.LATCH_LATCHED
+                            updated_latch_entry[self.ANALOG_LATCHED_DATA] = value
+                            # time stamp it
+                            updated_latch_entry[self.ANALOG_TIME_STAMP] = time.time()
+                            self.analog_latch_table[pin] = updated_latch_entry
                     else:
                         pass  # haven't hit target
                 elif latching_entry[self.LATCHED_THRESHOLD_TYPE] == self.ANALOG_LATCH_GTE:
                     if value >= latching_entry[self.ANALOG_LATCH_DATA_TARGET]:
-                        updated_latch_entry = latching_entry
-                        updated_latch_entry[self.LATCH_STATE] = self.LATCH_LATCHED
-                        updated_latch_entry[self.ANALOG_LATCHED_DATA] = value
-                        # time stamp it
-                        updated_latch_entry[self.ANALOG_TIME_STAMP] = time.time()
-                        self.analog_latch_table[pin] = updated_latch_entry
+                        if latching_entry[self.ANALOG_LATCH_CALLBACK] is not None:
+                            self.analog_latch_table[pin] = [0, 0, 0, 0, 0, None]
+                            latching_entry[self.ANALOG_LATCH_CALLBACK]([self.pymata.ANALOG | self.pymata.LATCH_MODE,
+                                                                        pin, value, time.time()])
+                        else:
+                            updated_latch_entry = latching_entry
+                            updated_latch_entry[self.LATCH_STATE] = self.LATCH_LATCHED
+                            updated_latch_entry[self.ANALOG_LATCHED_DATA] = value
+                            # time stamp it
+                            updated_latch_entry[self.ANALOG_TIME_STAMP] = time.time()
+                            self.analog_latch_table[pin] = updated_latch_entry
                     else:
                         pass  # haven't hit target:
                 elif latching_entry[self.LATCHED_THRESHOLD_TYPE] == self.ANALOG_LATCH_LT:
                     if value < latching_entry[self.ANALOG_LATCH_DATA_TARGET]:
-                        updated_latch_entry = latching_entry
-                        updated_latch_entry[self.LATCH_STATE] = self.LATCH_LATCHED
-                        updated_latch_entry[self.ANALOG_LATCHED_DATA] = value
-                        # time stamp it
-                        updated_latch_entry[self.ANALOG_TIME_STAMP] = time.time()
-                        self.analog_latch_table[pin] = updated_latch_entry
+                        if latching_entry[self.ANALOG_LATCH_CALLBACK] is not None:
+                            latching_entry[self.ANALOG_LATCH_CALLBACK]([self.pymata.ANALOG | self.pymata.LATCH_MODE,
+                                                                        pin, value, time.time()])
+                            self.analog_latch_table[pin] = [0, 0, 0, 0, 0, None]
+                        else:
+                            updated_latch_entry = latching_entry
+                            updated_latch_entry[self.LATCH_STATE] = self.LATCH_LATCHED
+                            updated_latch_entry[self.ANALOG_LATCHED_DATA] = value
+                            # time stamp it
+                            updated_latch_entry[self.ANALOG_TIME_STAMP] = time.time()
+                            self.analog_latch_table[pin] = updated_latch_entry
                     else:
                         pass  # haven't hit target:
                 elif latching_entry[self.LATCHED_THRESHOLD_TYPE] == self.ANALOG_LATCH_LTE:
                     if value <= latching_entry[self.ANALOG_LATCH_DATA_TARGET]:
-                        updated_latch_entry = latching_entry
-                        updated_latch_entry[self.LATCH_STATE] = self.LATCH_LATCHED
-                        updated_latch_entry[self.ANALOG_LATCHED_DATA] = value
-                        # time stamp it
-                        updated_latch_entry[self.ANALOG_TIME_STAMP] = time.time()
-                        self.analog_latch_table[pin] = updated_latch_entry
+                        if latching_entry[self.ANALOG_LATCH_CALLBACK] is not None:
+                            latching_entry[self.ANALOG_LATCH_CALLBACK]([self.pymata.ANALOG | self.pymata.LATCH_MODE,
+                                                                        pin, value, time.time()])
+                            self.analog_latch_table[pin] = [0, 0, 0, 0, 0, None]
+                        else:
+                            updated_latch_entry = latching_entry
+                            updated_latch_entry[self.LATCH_STATE] = self.LATCH_LATCHED
+                            updated_latch_entry[self.ANALOG_LATCHED_DATA] = value
+                            # time stamp it
+                            updated_latch_entry[self.ANALOG_TIME_STAMP] = time.time()
+                            self.analog_latch_table[pin] = updated_latch_entry
                     else:
                         pass  # haven't hit target:
                 else:
@@ -423,8 +486,10 @@ class PyMataCommandHandler(threading.Thread):
         This method handles the incoming digital message.
         It stores the data values in the digital response table.
         Data is stored for all 8 bits of a  digital port
-        @param data: Message data from Firmata
-        @return: No return value.
+
+        :param data: Message data from Firmata
+
+        :return: No return value.
         """
         port = data[0]
         port_data = (data[self.MSB] << 7) + data[self.LSB]
@@ -432,29 +497,52 @@ class PyMataCommandHandler(threading.Thread):
         # set all the pins for this reporting port
         # get the first pin number for this report
         pin = port * 8
-        for pin in range(pin, pin + 8):
+        for pin in range(pin, min(pin + 8, self.total_pins_discovered)):
             # shift through all the bit positions and set the digital response table
             with self.pymata.data_lock:
+                # look at the previously stored value for this pin
+                prev_data = self.digital_response_table[pin][self.RESPONSE_TABLE_PIN_DATA_VALUE]
+                # get the current value
                 self.digital_response_table[pin][self.RESPONSE_TABLE_PIN_DATA_VALUE] = port_data & 0x01
+                # if the values differ and callback is enabled for the pin, then send out the callback
+                if prev_data != port_data & 0x01:
+                    callback = self.digital_response_table[pin][self.RESPONSE_TABLE_CALLBACK]
+                    if callback:
+                        callback([self.pymata.DIGITAL, pin,
+                                  self.digital_response_table[pin][self.RESPONSE_TABLE_PIN_DATA_VALUE]])
+
                 # determine if the latch data table needs to be updated for each pin
                 latching_entry = self.digital_latch_table[pin]
                 if latching_entry[self.LATCH_STATE] == self.LATCH_ARMED:
                     if latching_entry[self.LATCHED_THRESHOLD_TYPE] == self.DIGITAL_LATCH_LOW:
                         if (port_data & 0x01) == 0:
-                            updated_latch_entry = latching_entry
-                            updated_latch_entry[self.LATCH_STATE] = self.LATCH_LATCHED
-                            updated_latch_entry[self.DIGITAL_LATCHED_DATA] = self.DIGITAL_LATCH_LOW
-                            # time stamp it
-                            updated_latch_entry[self.DIGITAL_TIME_STAMP] = time.time()
+                            if latching_entry[self.DIGITAL_LATCH_CALLBACK] is not None:
+                                self.digital_latch_table[pin] = [0, 0, 0, 0, None]
+                                latching_entry[self.DIGITAL_LATCH_CALLBACK](
+                                    [self.pymata.OUTPUT | self.pymata.LATCH_MODE,
+                                     pin, 0, time.time()])
+
+                            else:
+                                updated_latch_entry = latching_entry
+                                updated_latch_entry[self.LATCH_STATE] = self.LATCH_LATCHED
+                                updated_latch_entry[self.DIGITAL_LATCHED_DATA] = self.DIGITAL_LATCH_LOW
+                                # time stamp it
+                                updated_latch_entry[self.DIGITAL_TIME_STAMP] = time.time()
                         else:
                             pass
                     elif latching_entry[self.LATCHED_THRESHOLD_TYPE] == self.DIGITAL_LATCH_HIGH:
                         if port_data & 0x01:
-                            updated_latch_entry = latching_entry
-                            updated_latch_entry[self.LATCH_STATE] = self.LATCH_LATCHED
-                            updated_latch_entry[self.DIGITAL_LATCHED_DATA] = self.DIGITAL_LATCH_HIGH
-                            # time stamp it
-                            updated_latch_entry[self.DIGITAL_TIME_STAMP] = time.time()
+                            if latching_entry[self.DIGITAL_LATCH_CALLBACK] is not None:
+                                self.digital_latch_table[pin] = [0, 0, 0, 0, None]
+                                latching_entry[self.DIGITAL_LATCH_CALLBACK](
+                                    [self.pymata.OUTPUT | self.pymata.LATCH_MODE,
+                                     pin, 1, time.time()])
+                            else:
+                                updated_latch_entry = latching_entry
+                                updated_latch_entry[self.LATCH_STATE] = self.LATCH_LATCHED
+                                updated_latch_entry[self.DIGITAL_LATCHED_DATA] = self.DIGITAL_LATCH_HIGH
+                                # time stamp it
+                                updated_latch_entry[self.DIGITAL_TIME_STAMP] = time.time()
                         else:
                             pass
                 else:
@@ -466,35 +554,54 @@ class PyMataCommandHandler(threading.Thread):
     def encoder_data(self, data):
         """
         This method handles the incoming encoder data message and stores
-        the data in the response table.
-        @param data: Message data from Firmata
-        @return: No return value.
+        the data in the digital response table.
+
+        :param data: Message data from Firmata
+
+        :return: No return value.
         """
+        prev_val = self.digital_response_table[data[self.RESPONSE_TABLE_MODE]][self.RESPONSE_TABLE_PIN_DATA_VALUE]
         val = int((data[self.MSB] << 7) + data[self.LSB])
         # set value so that it shows positive and negative values
         if val > 8192:
             val -= 16384
+        pin = data[0]
         with self.pymata.data_lock:
             self.digital_response_table[data[self.RESPONSE_TABLE_MODE]][self.RESPONSE_TABLE_PIN_DATA_VALUE] = val
+            if prev_val != val:
+                callback = self.digital_response_table[pin][self.RESPONSE_TABLE_CALLBACK]
+                if callback is not None:
+                    callback([self.pymata.ENCODER, pin,
+                              self.digital_response_table[pin][self.RESPONSE_TABLE_PIN_DATA_VALUE]])
 
     def sonar_data(self, data):
         """
         This method handles the incoming sonar data message and stores
         the data in the response table.
-        @param data: Message data from Firmata
-        @return: No return value.
+
+        :param data: Message data from Firmata
+
+        :return: No return value.
         """
         val = int((data[self.MSB] << 7) + data[self.LSB])
         pin_number = data[0]
         with self.pymata.data_lock:
-            self.active_sonar_map[pin_number] = val
+            sonar_pin_entry = self.active_sonar_map[pin_number]
             # also write it into the digital response table
             self.digital_response_table[data[self.RESPONSE_TABLE_MODE]][self.RESPONSE_TABLE_PIN_DATA_VALUE] = val
+            # send data through callback if there is a callback function for the pin
+            if sonar_pin_entry[0] is not None:
+                # check if value changed since last reading
+                if sonar_pin_entry[1] != val:
+                    self.active_sonar_map[pin_number][0]([self.pymata.SONAR, pin_number, val])
+            # update the data in the table with latest value
+            sonar_pin_entry[1] = val
+            self.active_sonar_map[pin_number] = sonar_pin_entry
 
     def get_analog_response_table(self):
         """
         This method returns the entire analog response table to the caller
-        @return: The analog response table.
+        :return: The analog response table.
         """
         with self.pymata.data_lock:
             data = self.analog_response_table
@@ -503,7 +610,7 @@ class PyMataCommandHandler(threading.Thread):
     def get_digital_response_table(self):
         """
         This method returns the entire digital response table to the caller
-        @return: The digital response table.
+        :return: The digital response table.
         """
         with self.pymata.data_lock:
             data = self.digital_response_table
@@ -513,9 +620,11 @@ class PyMataCommandHandler(threading.Thread):
         """
         This method will send a Sysex command to Firmata with any accompanying data
 
-        @param sysex_command: sysex command
-        @param sysex_data: data for command
-        @return : No return value.
+        :param sysex_command: sysex command
+
+        :param sysex_data: data for command
+
+        :return : No return value.
         """
         if not sysex_data:
             sysex_data = []
@@ -534,8 +643,10 @@ class PyMataCommandHandler(threading.Thread):
     def send_command(self, command):
         """
         This method is used to transmit a non-sysex command.
-        @param command: Command to send to firmata includes command + data formatted by caller
-        @return : No return value.
+
+        :param command: Command to send to firmata includes command + data formatted by caller
+
+        :return : No return value.
         """
         send_message = ""
         for i in command:
@@ -548,7 +659,8 @@ class PyMataCommandHandler(threading.Thread):
         """
         Send the reset command to the Arduino.
         It resets the response tables to their initial values
-        @return: No return value
+
+        :return: No return value
         """
         data = chr(self.SYSTEM_RESET)
         self.pymata.transport.write(data)
@@ -565,50 +677,64 @@ class PyMataCommandHandler(threading.Thread):
 
             # reinitialize tables
             for pin in range(0, self.total_pins_discovered):
-                response_entry = [self.pymata.INPUT, 0]
+                response_entry = [self.pymata.INPUT, 0, None]
                 self.digital_response_table.append(response_entry)
 
             for pin in range(0, self.number_of_analog_pins_discovered):
-                response_entry = [self.pymata.INPUT, 0]
+                response_entry = [self.pymata.INPUT, 0, None]
                 self.analog_response_table.append(response_entry)
 
-    #noinspection PyMethodMayBeStatic
+    # noinspection PyMethodMayBeStatic
     # keeps pycharm happy
     def _string_data(self, data):
         """
         This method handles the incoming string data message from Firmata.
         The string is printed to the console
 
-        @param data: Message data from Firmata
-        @return: No return value.s
+        :param data: Message data from Firmata
+
+        :return: No return value.s
         """
-        print "_string_data:"
+        print("_string_data:")
         string_to_print = []
         for i in data[::2]:
             string_to_print.append(chr(i))
-        print string_to_print
+        print("".join(string_to_print))
 
     def i2c_reply(self, data):
         """
         This method receives replies to i2c_read requests. It stores the data for each i2c device
         address in a dictionary called i2c_map. The data is retrieved via a call to i2c_get_read_data()
         in pymata.py
-        @param data: raw data returned from i2c device
+        It a callback was specified in pymata.i2c_read, the raw data is sent through the callback
+
+        :param data: raw data returned from i2c device
         """
+
         reply_data = []
         address = (data[0] & 0x7f) + (data[1] << 7)
         register = data[2] & 0x7f + data[3] << 7
         reply_data.append(register)
-        for i in xrange(4, len(data), 2):
+        for i in range(4, len(data), 2):
             data_item = (data[i] & 0x7f) + (data[i + 1] << 7)
             reply_data.append(data_item)
-        self.i2c_map[address] = reply_data
+        # retrieve the data entry for this address from the i2c map
+        if address in self.i2c_map:
+            i2c_data = self.i2c_map.get(address, None)
+
+            i2c_data[1] = reply_data
+            self.i2c_map[address] = i2c_data
+            # is there a call back for this entry?
+            # if yes, return a list of bytes through the callback
+            if i2c_data[0] is not None:
+                i2c_data[0]([self.pymata.I2C, address, reply_data])
 
     def capability_response(self, data):
         """
         This method handles a capability response message and stores the results to be retrieved
         via get_capability_query_results() in pymata.py
-        @param data: raw capability data
+
+        :param data: raw capability data
         """
         self.capability_query_results = data
 
@@ -616,7 +742,8 @@ class PyMataCommandHandler(threading.Thread):
         """
         This method handles a pin state response message and stores the results to be retrieved
         via get_pin_state_query_results() in pymata.py
-        @param data:  raw pin state data
+
+        :param data:  raw pin state data
         """
         self.last_pin_query_results = data
 
@@ -624,7 +751,8 @@ class PyMataCommandHandler(threading.Thread):
         """
         This method handles an analog mapping query response message and stores the results to be retrieved
         via get_analog_mapping_request_results() in pymata.py
-        @param data: raw analog mapping data
+
+        :param data: raw analog mapping data
         """
         self.analog_mapping_query_results = data
 
@@ -692,7 +820,8 @@ class PyMataCommandHandler(threading.Thread):
                             # go to the beginning of the loop to process the next command
                     continue
 
-                #is this a command byte in the range of 0x80-0xff - these are the non-sysex messages
+                # is this a command byte in the range of 0x80-0xff - these are the non-sysex messages
+
                 elif 0x80 <= data <= 0xff:
                     # look up the method for the command in the command dispatch table
                     # for the digital reporting the command value is modified with port number
@@ -718,21 +847,20 @@ class PyMataCommandHandler(threading.Thread):
                     # get the number of parameters that this command provides
                     num_args = dispatch_entry[1]
 
-                    #look at the number of args that the selected method requires
+                    # look at the number of args that the selected method requires
                     # now get that number of bytes to pass to the called method
                     for i in range(num_args):
                         while len(self.pymata.command_deque) == 0:
                             pass
                         data = self.pymata.command_deque.popleft()
                         command_data.append(data)
-                        #go execute the command with the argument list
+                        # go execute the command with the argument list
                     method(command_data)
 
                     # go to the beginning of the loop to process the next command
                     continue
-
-
-
+            else:
+                time.sleep(.1)
 
 
 
